use std::mem;
use std::rc::Rc;
use std::cell::RefCell;

mod add_two_numbers;
mod array;
mod binary_search;
mod bt;
mod tree;
mod stack;
mod container_with_most_water;
mod dp;
mod four_sum;
mod integer_to_roman;
mod letter_combinations_of_a_phone_number;
mod list;
mod longest_common_prefix;
mod longest_palindromic_substring;
mod longest_substring_without_repeating_characters;
mod move_zeros;
mod other;
mod palindrome_number;
mod recrusion;
mod regular_expression_matching;
mod remove_nth_node_from_end_of_list;
mod reverse_integer;
mod roman_to_integer;
mod str;
mod swap_nodes_in_pairs;
mod three_sum;
mod three_sum_closest;
mod two_sum;
mod valid_parenthese;
mod zigzag_conversion;

// Definition for singly-linked list.
#[derive(PartialEq, Eq, Clone, Debug)]
pub struct ListNode {
    pub val: i32,
    pub next: Option<Box<ListNode>>,
}

impl ListNode {
    #[inline]
    fn new(val: i32) -> Self {
        ListNode { next: None, val }
    }
}

pub fn to_list(vector: Vec<i32>) -> Option<Box<ListNode>> {
    let mut head = None;
    for &val in vector.iter().rev() {
        let mut new_node = ListNode::new(val);
        new_node.next = head;
        head = Some(Box::new(new_node))
    }

    head
}

pub fn to_vec(mut head: Option<Box<ListNode>>) -> Vec<i32> {
    let mut v = vec![];
    while let Some(head_p) = head {
        v.push(head_p.as_ref().val);
        head = head_p.next;
    }
    v
}

pub fn reverse_list(head: Option<Box<ListNode>>) -> Option<Box<ListNode>> {
    let mut head = head;
    let mut dummy: Option<Box<ListNode>> = None;
    let p = &mut dummy;
    let cur = &mut head;
    while cur.is_some() {
        mem::swap(p, &mut cur.as_mut().unwrap().next);
        mem::swap(p, cur);
        // println!("{:?}", to_vec(p.clone()))
    }

    dummy
}


// Definition for a binary tree node.
#[derive(Debug, PartialEq, Eq)]
pub struct TreeNode {
  pub val: i32,
  pub left: Option<Rc<RefCell<TreeNode>>>,
  pub right: Option<Rc<RefCell<TreeNode>>>,
}

impl TreeNode {
  #[inline]
  pub fn new(val: i32) -> Self {
    TreeNode {
      val,
      left: None,
      right: None
    }
  }
}


fn build_tree(
    nums: &Vec<Option<i32>>,
    index: usize,
) -> Option<Rc<RefCell<TreeNode>>> {
    if index >= nums.len() {
        return None;
    }

    if let Some(val) = nums[index] {
        let node = Rc::new(RefCell::new(TreeNode::new(val)));
        node.borrow_mut().left = build_tree(nums, 2 * index + 1);
        node.borrow_mut().right = build_tree(nums, 2 * index + 2);
        Some(node)
    } else {
        None
    }



}


pub fn vec_to_tree(v: Vec<Option<i32>>) -> Option<Rc<RefCell<TreeNode>>> {
    if v.is_empty() { return None; }
    build_tree(&v, 0)
}


fn traversal(root: Option<Rc<RefCell<TreeNode>>>, res: &mut Vec<i32>) {
    if let Some(node) = root {
        traversal(node.borrow().left.clone(), res);
        res.push(node.borrow().val);
        traversal(node.borrow().right.clone(), res);
    }
}

pub fn inorder_traversal(root: Option<Rc<RefCell<TreeNode>>>) -> Vec<i32> {
    let mut res = vec![];
    traversal(root, &mut res);
    res

}


pub fn ctoi(c: char) -> usize {
    c as usize - '0' as usize
}

pub fn itoc(i: usize) -> char {
    (i as u8 + b'0') as char
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn it_works() {
        let v = vec![1, 2, 3];
        let head = to_list(v).unwrap();
        println!("{:?}", head.as_ref());
        let vv = to_vec(Some(head));
        println!("vv: {:?}", vv);
        assert_eq!(1, 1);
    }

    #[test]
    fn test_reverse_list() {
        let v = vec![1, 2, 3];
        let head = to_list(v);
        let reverse_head = reverse_list(head);

        assert_eq!(vec![3, 2, 1], to_vec(reverse_head));
    }

    #[test]
    fn test_reverse_list2() {
        let v = vec![1];
        let head = to_list(v);
        let reverse_head = reverse_list(head);
        assert_eq!(vec![1], to_vec(reverse_head));
    }

    #[test]
    fn test_inorder_traversal() {
        let nums = vec![Some(1),Some(2),Some(3),Some(4),Some(5)];
        let root = vec_to_tree(nums);
        let res = inorder_traversal(root);
        assert_eq!(res, vec![4,2,5,1,3]);
    }

    #[test]
    fn test_ctoi() {
        let c = '3';
        let res = ctoi(c);
        assert_eq!(res, 3);
    }

    #[test]
    fn test_itoc() {
        let i = 3;
        let res = itoc(i);
        assert_eq!(res, '3');
    }
}
